/*
 *  This file contains Good Sample Code subject to the Good Dynamics SDK Terms and Conditions.
 *  (c) 2016 BlackBerry Limited. All rights reserved.
 */


#import <Foundation/Foundation.h>
#import "RSSFeed.h"

/*
  
 This RSS Manager is a singleton that looks after RSS Settings and
 manages the available feeds list.
 
 It handles the secure store and has two NSNotifications to let
 observers know when a feed is added or the feed mode is changed
 
 */




//The two supported request modes
typedef NS_ENUM(NSInteger, RequestMode) {
    RequestModeNSURL = 0,
    RequestModeGDHttp = 1,
    RequestModeNSURLSession = 2
    };

//An NSNotification that is posted when a new feed is sucessfully added
#define kRSSFeedAddedNotification @"kRSSFeedAddedNotification"

//An NSNotification that is posted when the request mode is changed - could have used KVO
#define kRSSRequestModeChangedNotification @"kRSSRequestModeChangedNotification"


@interface RSSManager : NSObject

+(RSSManager*)sharedRSSManager;


//Adds a new feed to the array and returns success / fail boolean
-(BOOL)saveFeed:(RSSFeed*)rssFeed withName:(NSString*)feedName andURLString:(NSString*)urlString;

//Removes a feed from the array (and this controller handles secure storage management)
-(void)removeFeedAtPosition:(NSInteger)pos;

//Accessors to the private array
-(int)numberOfFeeds;
-(RSSFeed*)feedAtPos:(NSInteger)pos;

//Allow access to the request mode
-(RequestMode)currentRequestMode;
-(void)changeCurrentRequestMode:(RequestMode)_requestMode;


@end
