/*
 *  This file contains Good Sample Code subject to the Good Dynamics SDK Terms and Conditions.
 *  (c) 2013 Good Technology Corporation. All rights reserved.
 */

#import "AddFeedViewController.h"
#import "UIAccessibilityIdentifiers.h"
#import "RSSManager.h"

@interface AddFeedViewController () <UITextFieldDelegate>

@property(strong, nonatomic) IBOutlet UITextField *feedNameTextField;
@property(strong, nonatomic) IBOutlet UITextField *feedURLTextField;
@property(strong, nonatomic) IBOutlet UIButton *addFeedButton;

@end

@implementation AddFeedViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // UI Testing initialization
    self.view.accessibilityIdentifier = BBDAddFeedViewID;
    self.feedNameTextField.accessibilityIdentifier = BBDFeedNameFieldID;
    self.feedURLTextField.accessibilityIdentifier = BBDFeedURLFieldID;
    self.addFeedButton.accessibilityIdentifier = BBDAddFeedButtonID;
    
    // setting dismissing bar button item for iPhone
    if (self.traitCollection.userInterfaceIdiom == UIUserInterfaceIdiomPhone) {
        self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemStop
                                                                                               target:self
                                                                                               action:@selector(closeView)];
    }
    
    //Update fields
    [self updateAccordingEditingState];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    [self.feedNameTextField becomeFirstResponder];
}

- (IBAction)tapOnView:(UITapGestureRecognizer *)sender
{
    [self.view endEditing:YES];
}

- (void)updateAccordingEditingState
{
    if(self.editRSSFeed)
    {
        //Title for iPad
        [self setTitle:@"Edit RSS Feed"];
        [self.addFeedButton setTitle:@"Update Feed"
                            forState:UIControlStateNormal];
        [self.feedURLTextField setText:self.editRSSFeed.rssUrl.absoluteString];
        [self.feedNameTextField setText:self.editRSSFeed.rssName];
    } else {
        [self setTitle:@"Add RSS Feed"];
        [self.addFeedButton setTitle:@"Add Feed"
                            forState:UIControlStateNormal];
        [self.feedURLTextField setText:@""];
        [self.feedNameTextField setText:@""];
    }
}

#pragma mark - actions
//For iPhone buttons
-(void)closeView
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

-(IBAction)addFeed:(id)sender
{
    
    //Ask the RSSController to add the feed to secure storage
    if([[RSSManager sharedRSSManager] saveFeed:self.editRSSFeed withName:self.feedNameTextField.text andURLString:self.feedURLTextField.text])
    {
    	if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
        {
        	//Success - just dismiss this view
        	[self closeView];
        } else {
            //Clear the fields
            self.editRSSFeed = nil;
            [self updateAccordingEditingState];
            [self.feedNameTextField resignFirstResponder];
            [self.feedURLTextField resignFirstResponder];
        }
    } else {
    	//There was a problem adding the feed
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Feed Insert Error" message:@"There was a problem with the new feed details\nPlease check that you have entered a name for the feed and a valid URL" preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *ok = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil];
        [alert addAction:ok];
        [self presentViewController:alert animated:YES completion:nil];
    }
}


#pragma mark - text field delegate

//This allows the user to tab through to the Next UITextField
-(BOOL)textFieldShouldReturn:(UITextField*)textField;
{
    NSInteger nextTag = textField.tag + 1;
    // Try to find next responder
    UIResponder* nextResponder = [textField.superview viewWithTag:nextTag];
    if (nextResponder) {
        // Found next responder, so set it.
        [nextResponder becomeFirstResponder];
    } else {
        // Not found, so add feed
        [self addFeed:nil];
    }
    return NO; // We do not want UITextField to insert line-breaks.
}

@end
