/*
 * (c) 2017 BlackBerry Limited. All rights reserved.
 */

#import <XCTest/XCTest.h>
#import "BBDAutomatedTestSupport.h"
#import "UIAccessibilityIdentifiers.h"

static NSString* const BBDFeedName = @"Test feeld";
static NSString* const BBDFeedURL  = @"http://rss.cnn.com/rss/edition_world.rss";

@interface RSSReaderUITests : XCTestCase

@end

@implementation RSSReaderUITests

- (void)setUp
{
    // Put setup code here. This method is called before the invocation of each test method in the class.
    [[XCUIApplication new] terminate];
    [super setUp];
    
    // In UI tests it is usually best to stop immediately when a failure occurs.
    self.continueAfterFailure = NO;
}

- (void)tearDown
{
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
    [[XCUIApplication new] terminate];
}

- (void)testProvision
{
    XCUIApplication *application = [[XCUIApplication alloc] init];
    [application launch];
    
    BBDAutomatedTestSupport *ats = [BBDAutomatedTestSupport sharedInstance];
    
    [self fetchCredentials];
    
    [ats setupBBDAutomatedTestSupportWithApplication:application
                                         forTestCase:self];
    
    XCTAssertTrue([ats loginOrProvisionBBDApp], @"Activation Failed!");
    
    BOOL mainUIShown = [ats isScreenShown:BBDFeedListViewID timeout:10.f];
    
    XCTAssertTrue(mainUIShown, @"Main UI is not on screen");
}

- (void)testProvisionWithNoPassword
{
    XCUIApplication *application = [[XCUIApplication alloc] init];
    [application launch];
    
    BBDAutomatedTestSupport *ats = [BBDAutomatedTestSupport sharedInstance];
    
    [self fetchCredentials];
    
    [ats setupBBDAutomatedTestSupportWithApplication:application
                                         forTestCase:self];
    
    XCTAssertTrue([ats loginOrProvisionBBDAppWithAuthType:BBDNoPasswordAuthType], @"Activation Failed!");
    
    BOOL mainUIShown = [ats isScreenShown:BBDFeedListViewID timeout:20.f];
    
    XCTAssertTrue(mainUIShown, @"Main UI is not on screen");
}

- (void)testAddNewFeed
{
    XCUIApplication *application = [[XCUIApplication alloc] init];
    [application launch];
    
    BBDAutomatedTestSupport *ats = [BBDAutomatedTestSupport sharedInstance];
    
    [self fetchCredentials];
    
    [ats setupBBDAutomatedTestSupportWithApplication:application
                                         forTestCase:self];
    
    XCTAssertTrue([ats loginOrProvisionBBDApp], @"Activation Failed!");
    
    BOOL errorUIAlert = [ats isAlertShownWithStaticText:@"Message" timeout:10.f];
    
    if (errorUIAlert) {
        
        NSLog(@"Error allert on screen!");
        
        [ats tapOnItemOfType:XCUIElementTypeAlert containingText:@"OK"];
    }
    
    BOOL mainUIShown = [ats isScreenShown:BBDFeedListViewID timeout:10.f];
    
    XCTAssertTrue(mainUIShown, @"Main UI is not on screen");
    
    XCTAssertTrue([ats tapOnItemOfType:XCUIElementTypeButton withAccessID:BBDAddFeedBarButtonID timeout:10.f], @"Add button is not on screen!");
    
    BOOL addFeedUIShown = [ats isScreenShown:BBDAddFeedViewID timeout:10.f];
    
    XCTAssertTrue(addFeedUIShown, @"Add Feed UI is not on screen");
    
    XCTAssertTrue([ats enterText:BBDFeedName inViewOfType:XCUIElementTypeTextField withAccessID:BBDFeedNameFieldID timeout:10.f], @"Feed name typing failed");
    
    XCTAssertTrue([ats enterText:BBDFeedURL inViewOfType:XCUIElementTypeTextField withAccessID:BBDFeedURLFieldID timeout:10.f], @"Url to RSS Feed typing failed");
    
    XCTAssertTrue([ats tapOnItemOfType:XCUIElementTypeButton withAccessID:BBDAddFeedButtonID timeout:10.f], @"Add button typing failed!");
    
    BOOL mainUIShownAgain = [ats isScreenShown:BBDFeedListViewID timeout:10.f];
    
    XCTAssertTrue(mainUIShownAgain, @"Main UI is not on screen");

    XCTAssertTrue([ats tapOnRowWithStaticText:BBDFeedName timeout:10.f], @"Table row is not on screen!");
    
    BOOL rssListUIShown = [ats isScreenShown:BBDRssListViewID timeout:10.f];
    
    XCTAssertTrue(rssListUIShown, @"List of RSS is not on screen");
}

- (void)fetchCredentials
{
    NSString *credetialsJSONBundlePath = [[NSBundle bundleForClass:[self class]] pathForResource:@"credentials"
                                                                                          ofType:@"json"];
    [[BBDAutomatedTestSupport sharedInstance] fetchCredentialsFromFileAtPath:credetialsJSONBundlePath];
}

@end
